#!/bin/bash

set -e

# PREREQUISITES
DOCKER_SERVER_MIN_VER='1.11'
#DOCKER_SERVER_CURRENT_VER, this should be passed by the installer script [mission-control.sh] as $(docker version --format '{{.Server.Version}}') || DOCKER_SERVER_CURRENT_VER=""

MAX_OPEN_FILES_MIN_VALUE=10000
MAX_OPEN_FILES_REQUIRED_VALUE=32000

# JFMC_APP_SERVICES_LIST, the services names as they are in the docker-cmpose file, SPACE separated
JFMC_APP_SERVICES_LIST="server core"

# PREREQUISITES
DOCKER_SERVER_MIN_VER='1.11'
#DOCKER_SERVER_CURRENT_VER, this should be passed by the installer script [mission-control.sh] as $(docker version --format '{{.Server.Version}}') || DOCKER_SERVER_CURRENT_VER=""

aparse()
{
    if [ ! -z $1 ]; then
        command=$1
        shift
    else
        command=usage
        return 0
    fi

    if [[ $# > 0 ]] ; then
              ALL_ARGS="${ALL_ARGS}${1} "
    fi
}

if [ -z "$COMPOSE_PROJECT_NAME" ]; then
    COMPOSE_PROJECT_NAME="jfmc"
fi

# Installed version
INSTALLER_DATA_FOLDER="/data"
INSTALLED_VERSION_FILE="${INSTALLER_DATA_FOLDER}/version.current"
INSTALLED_VERSION_FILE_HISTORY="${INSTALLER_DATA_FOLDER}/version.history"
INSTALLED_VERSION=''
JFMC_GLOBAL_MOUNT_ROOT="/jfmc_global_mount_root" # This should not be used, unless special maintenance is needed

if [ -f $INSTALLED_VERSION_FILE ]
then
    INSTALLED_VERSION=$(cat $INSTALLED_VERSION_FILE)
fi

# Maintainer / Installer version
MAINTAINER_VERSION_FILE="/opt/jfrog/jfmc/installer/version.current"
MAINTAINER_VERSION=''
if [ -f $MAINTAINER_VERSION_FILE ]
then
    MAINTAINER_VERSION=$(cat $MAINTAINER_VERSION_FILE)
fi

export IMAGE_MOUNT_ROOT="/var/opt/jfrog/mission-control"
export JFMC_PORT

exitOnError()
{
    msg=$1
    echo "ERROR: ${msg}"
    exit 1
}

usage() {
    cat ./readme.txt
    exit 1
}

version_lt() { test "$(echo "$@" | tr " " "\n" | sort -rV | head -n 1)" != "$1"; }

# $1 should be WARNING or ERROR
printUlimitErrorMsg()
{
    local error_level=$1
    local number=$2
    MSG="
${error_level}: Max number of open files [${number}] is too low !!!

Add the following to /etc/security/limits.conf file on HOST:
* soft nofile ${MAX_OPEN_FILES_REQUIRED_VALUE}
* hard nofile ${MAX_OPEN_FILES_REQUIRED_VALUE}

Note: Also verify docker limits configuration if exists [usually in /etc/default/docker]

"

    case ${error_level} in
    "ERROR")
        echo -e "\033[31m${MSG}\033[0m "
    ;;
    "WARNING")
        echo -e "\033[33m${MSG}\033[0m"
    ;;
    esac

    echo
}

checkPrerequisites()
{
    #echo "Check dependencies:"

    if [ -z ${DOCKER_SERVER_CURRENT_VER} ] || [ "${DOCKER_SERVER_CURRENT_VER}" == "" ]
    then
        echo "WARNING: Docker daemon version is unknown"
        # Do not exit on warning
    fi

    if version_lt ${DOCKER_SERVER_CURRENT_VER} ${DOCKER_SERVER_MIN_VER}
    then
        echo ""
        echo "ERROR: Mission Control requires Docker daemon version ${DOCKER_SERVER_MIN_VER} or later. The current version is ${DOCKER_SERVER_CURRENT_VER}."
        return 1
    fi

    local max_map_count=$(cat /proc/sys/vm/max_map_count)
    local recommended_map_count=262144
    if [ $max_map_count -lt $recommended_map_count ]; then
        echo ""
        echo  "Not enough virtual memory to run Elastic search reliably. Recommended: [vm.max_map_count=$recommended_map_count]. \
Please refer to [https://www.elastic.co/guide/en/elasticsearch/reference/current/docker.html#docker-cli-run-prod-mode]"
        return 1
    fi


    # echo "Check max number of open files:"
    currentMaxOpenFilesVal=$(ulimit -n)

    if [ "${currentMaxOpenFilesVal}" != "unlimited" ] ; then

        # MAX_OPEN_FILES_REQUIRED_VALUE
        if [ ${currentMaxOpenFilesVal} -lt ${MAX_OPEN_FILES_REQUIRED_VALUE} ]
        then
            #echo "Trying to set max open files = ${MAX_OPEN_FILES_REQUIRED_VALUE}"
            ulimit -n ${MAX_OPEN_FILES_REQUIRED_VALUE} 2>/dev/null
            if [ $? -gt 0 ]
            then
                # Cannot set MAX_OPEN_FILES_REQUIRED_VALUE, will try to set MAX_OPEN_FILES_MIN_VALUE
                # MAX_OPEN_FILES_MIN_VALUE
                if [ ${currentMaxOpenFilesVal} -lt ${MAX_OPEN_FILES_MIN_VALUE} ]
                then
                    #echo "Trying to set max open files = ${MAX_OPEN_FILES_MIN_VALUE}"
                    ulimit -n ${MAX_OPEN_FILES_MIN_VALUE} 2>/dev/null
                    if [ $? -gt 0 ]; then
                        printUlimitErrorMsg "ERROR" "${currentMaxOpenFilesVal}"
                        return 1
                    fi
                fi
                printUlimitErrorMsg "WARNING" "${currentMaxOpenFilesVal}"
                # Do not exit on warning
            fi
        fi

    fi

    # dependencies check is passed
    return 0
}

runDockerCommand() {
    local action=$1

    case $action in
    "pull")
        if [ ! -z "$JFMC_DOCKER_REPO_LOGIN" ]; then
            docker login -u $JFMC_DOCKER_REPO_LOGIN -p $JFMC_DOCKER_REPO_PASSWORD $JFMC_DOCKER_REPO
        fi
    ;;
    "logs")
        action="logs -f"
    ;;
    "stop")
        # Stop Mission Control App layer first, than the DB layer
        docker-compose -p ${COMPOSE_PROJECT_NAME} ${action} ${JFMC_APP_SERVICES_LIST}
    ;;
    "stopApp")
        # Stop Mission Control App layer first, than the DB layer
        docker-compose -p ${COMPOSE_PROJECT_NAME} "stop" ${JFMC_APP_SERVICES_LIST}
        return $?
    ;;
    esac

    docker-compose -p ${COMPOSE_PROJECT_NAME} ${action}
}

# Returned code:
#0) $1=$2
#1) $1>$2
#2) $1<$2
verComp () {
    if [[ $1 == $2 ]]
    then
        return 0
    fi
    local IFS=.
    local i
    local ver1=($1)
    local ver2=($2)
    # fill empty fields in ver1 with zeros
    for ((i=${#ver1[@]}; i<${#ver2[@]}; i++))
    do
        ver1[i]=0
    done
    for ((i=0; i<${#ver1[@]}; i++))
    do
        if [[ -z ${ver2[i]} ]]
        then
            # fill empty fields in ver2 with zeros
            ver2[i]=0
        fi
        if ((10#${ver1[i]} > 10#${ver2[i]}))
        then
            return 1
        fi
        if ((10#${ver1[i]} < 10#${ver2[i]}))
        then
            return 2
        fi
    done
    return 0
}

isUpgradeAllowed() {

    local installedVersion=$INSTALLED_VERSION
    local maintainerVersion=$MAINTAINER_VERSION

    if [ -z $MAINTAINER_VERSION ]
    then
        exitOnError "MAINTAINER_VERSION is unknown"
    fi

    # This is a fresh installation
    if [ -z $INSTALLED_VERSION ]
    then
        return 0
    fi

    if [[ "$installedVersion" =~ ^b.* ]] && [[ "$maintainerVersion" =~ ^b.* ]]
    then
        installedVersion=($(echo $INSTALLED_VERSION | sed "s/^b//g"))
        maintainerVersion=($(echo $MAINTAINER_VERSION | sed "s/^b//g"))
    fi

    verComp $installedVersion $maintainerVersion 2>/dev/null
    if [ $? -eq 1 ]
    then
        return 1
    fi

    return 0
}

isCommandAllowed()
{
    local command=$1
    local installedVersion=$INSTALLED_VERSION
    local maintainerVersion=$MAINTAINER_VERSION

    case ${command} in
        "install"|"installLocal")
            [ -f $INSTALLED_VERSION_FILE ] && exitOnError "Mission Control is already installed. To upgrade, use: ./mission-control upgrade"
            checkPrerequisites || exitOnError "Prerequisites are not satisfied"
        ;;
        "upgrade")
            [ -f $INSTALLED_VERSION_FILE ] || exitOnError "Mission Control is not installed. To install, use: ./mission-control install"
            checkPrerequisites || exitOnError "Prerequisites are not satisfied"
            isUpgradeAllowed || exitOnError "Unable to upgrade to an older version"
        ;;
        "version")

        ;;
        "info")

        ;;
        *)
            # Is installed
            [ -f $INSTALLED_VERSION_FILE ] || exitOnError "Mission Control is not installed. To install, use: ./mission-control install"
            # Is correct version
            [ "${installedVersion}" == "${maintainerVersion}" ] || exitOnError "You are using the Mission Control installer version [${maintainerVersion}] which is different from the currently installed version [${installedVersion}]. Consider upgrading your version of Mission Control using: ./mission-control upgrade"
        ;;

    esac

    return 0
}

setVersions()
{
    if [ ! -f ${MAINTAINER_VERSION_FILE} ]
    then
        return 1
    fi

    cat ${MAINTAINER_VERSION_FILE} > ${INSTALLED_VERSION_FILE}
    cat ${MAINTAINER_VERSION_FILE} >> ${INSTALLED_VERSION_FILE_HISTORY}

    echo "${INSTALLED_VERSION_FILE}: $(cat ${INSTALLED_VERSION_FILE})"

    return 0
}

createQuartzUsers(){
    docker cp insight/postgres "${COMPOSE_PROJECT_NAME}_postgres_1":/ && \
    docker exec -i "${COMPOSE_PROJECT_NAME}_postgres_1" /postgres/createUsers.sh 
}

createESIndices(){
    docker cp insight/elasticsearch "${COMPOSE_PROJECT_NAME}_elasticsearch_1":/ && \
    docker exec -i "${COMPOSE_PROJECT_NAME}_elasticsearch_1" /elasticsearch/createIndices.sh
}

createMongoUsers(){
    USERS_CREATED_FILE="${INSTALLER_DATA_FOLDER}/mongo_users.created"
    if [ ! -f "${USERS_CREATED_FILE}" ];then
        docker cp mongodb "${COMPOSE_PROJECT_NAME}_mongodb_1":/ && \
        docker exec -i "${COMPOSE_PROJECT_NAME}_mongodb_1" /mongodb/createMongoUsers.sh
        touch $USERS_CREATED_FILE || exitOnError "Failed to create MongoDB default users"
    fi
}


createUsers(){
    createMongoUsers
    createQuartzUsers
    createESIndices
}

executeMigrationIfNeeded()
{
    local migrationName=$1
    local migrationForVersion=$2
    echo -e "\nINFO: Checking migration from [${INSTALLED_VERSION}] to [${migrationForVersion}]"
    verComp $INSTALLED_VERSION $migrationForVersion 2>/dev/null
    if [ $? -eq 2 ]
    then
        echo -e "\nINFO: Executing migration from [${migrationName}]"
        eval ${migrationName}
    fi
}

# backward compatible issues
# Mission Control should be stopped before this process
backwardCmpatibleProcess()
{
    echo -e "\nINFO: Executing possible migration steps"
    executeMigrationIfNeeded "migrateToSingleMount" "2"
    executeMigrationIfNeeded "genInternalCerts" "2"
}

migrateToSingleMount()
{
    local JFMC_MOUNT_V1="${JFMC_GLOBAL_MOUNT_ROOT}"
    local JFMC_MOUNT_V2="${JFMC_GLOBAL_MOUNT_ROOT}/jfmc"
    echo -e "\nINFO: Moving data,etc,logs from [${JFMC_MOUNT_V1}] to [${JFMC_MOUNT_V2}]"

    if [ ! -d ${JFMC_MOUNT_V2} ]; then
        mkdir -p ${JFMC_MOUNT_V2} && \
        echo "\nINFO: New folder at [${JFMC_MOUNT_V2}] was created"
    fi

    for folderToMigrate in "logs" "etc" "data"
    do
        local folderToMigrateV1="${JFMC_MOUNT_V1}/${folderToMigrate}"
        local folderToMigrateV2="${JFMC_MOUNT_V2}/${folderToMigrate}"
        if [ -d "${folderToMigrateV1}" ] && [ ! -d "${folderToMigrateV2}" ]
        then
            mv ${folderToMigrateV1} ${folderToMigrateV2} && \
            echo -e "\nINFO: ${folderToMigrate} folder renamed was:[${folderToMigrateV1}] new:[${folderToMigrateV2}]"
        fi
    done
}

genInternalCerts()
{
    echo -e "\nINFO: Generating internal certificates"
    local jfmcEtc=${JFMC_GLOBAL_MOUNT_ROOT}/jfmc/etc
    local jfmcSecurity=${jfmcEtc}/security/
    local insightSecurity=${JFMC_GLOBAL_MOUNT_ROOT}/insight/etc/security/
    for folder in ${jfmcSecurity} ${insightSecurity}
    do
        if [ ! -d ${folder} ]; then
            mkdir -p ${folder}
        fi
    done
    local storePassword=$(cat /dev/urandom | env LC_CTYPE=C tr -cd 'a-f0-9' | head -c 16)
    ./genCerts.sh ${storePassword} ${jfmcSecurity} ${insightSecurity} && \
    echo ${storePassword} > ${jfmcSecurity}/passwd && \
    echo ${storePassword} > ${insightSecurity}/passwd
}

migrateToElastic()
{
    # Create temp folder
    local influxDir=${INSTALLER_DATA_FOLDER}/influxmigration/csv
    mkdir -p ${influxDir}
    local influxCmd="docker exec -ti ${COMPOSE_PROJECT_NAME}_influxdb_1 influx"

    # attach jfmc_installer container to the jfmc_default network - to call elasticsearch url for migration 
    docker network connect jfmc_default jfmc_installer

    cd influxmigration
    echo -e "\nINFO: Exporting influxdb data as csv"
    ./exportInflux.sh "${influxCmd}" "${influxDir}"
    echo -e "\nINFO: Importing csv to elastic"
    ./migrate --csvpath "${influxDir}" --esurl "http://elasticsearch:9200/" --username elastic --password changeme --workingdir ${INSTALLER_DATA_FOLDER}/influxmigration/
}

##################################
############## MAIN ##############
##################################

aparse $@

isCommandAllowed ${command}

case $command in
    installLocal)
        ( setVersions \
        ) || exitOnError "Installation failed"
        echo -e "\nINFO: Mission Control was successfully installed. Run: ./mission-control start"
        ;;
    install)
        ( runDockerCommand pull && \
          genInternalCerts && \
          setVersions \
        ) || exitOnError "Installation failed"
        echo -e "\nINFO: Mission Control was successfully installed. Run: ./mission-control start"
        ;;
    upgrade)
        ( runDockerCommand pull && \
          runDockerCommand stopApp && \
          backwardCmpatibleProcess && \
          setVersions && \
          echo -e "\nINFO: Mission Control was successfully upgraded. Run: ./mission-control start" \
        ) || exitOnError "Upgrade failed"
        ;;
    upgradeLocal)
        ( runDockerCommand stopApp && \
          setVersions && \
          echo -e "\nINFO: Mission Control was successfully upgraded. Run: ./mission-control start" \
        ) || exitOnError "Upgrade failed"
        ;;
    generateCerts)
        ( genInternalCerts && \
          echo -e "\nINFO: Mission Control internal certs were successfully configured." \
        ) || exitOnError "Generating certificates failed"
        ;;
    migrateToElastic)
        ( migrateToElastic && \
          echo -e "\nINFO: Migration from InfluxDB to Elastic was successful." \
        ) || exitOnError "Migration from InfluxDB to Elastic failed"
        ;;
    start)
        runDockerCommand "up -d"
        createUsers
        echo -e "\nINFO: Mission Control was successfully started."
        ;;
    ps|logs|stop|kill|restart)
        runDockerCommand ${command}
        ;;
    version)
        if [ -f $INSTALLED_VERSION_FILE ]; then
            echo "Mission Control version: ${INSTALLED_VERSION}"
        else
            echo "Mission Control is not installed"
        fi
        ;;
    x-compose)
            xComposeCmd="docker-compose -p ${COMPOSE_PROJECT_NAME} ${ALL_ARGS}"
            echo "Advanced user: Going to execute '${xComposeCmd}'"
            ${xComposeCmd}
        ;;
    info)
           echo "Mission Control info:"
           echo "project name: ${COMPOSE_PROJECT_NAME}"
           echo "docker repo: ${JFMC_DOCKER_REPO}"
           echo "docker mount folder: ${JFMC_MOUNT_ROOT}"
        ;;
    *)
        usage
        ;;
esac
