#!/bin/bash

set -e

IS_LOCAL=false

# Parsing command line argument -u and -p to authenticate before running the installer
# and while pulling inside the installer
# Usage : ./mission-control.sh -u $USER -p $PASSWORD install
# Additionally, passing -i will allow the user to choose which version to download interactively
OPTIND=1
while getopts ":u:p:i" opt; do
  case $opt in
    u)
      JFMC_DOCKER_REPO_LOGIN=$OPTARG
      IS_LOCAL=true
      ;;
    p)
      JFMC_DOCKER_REPO_PASSWORD=$OPTARG
      IS_LOCAL=true
      ;;
   i)
      JFI_INTERACTIVE=true
      IS_LOCAL=true
      ;;  
  esac
done
shift "$((OPTIND-1))"

CMD="$@"

setUp() {
    JFMC_VERSION="%%myVersion%%"
    #During development, default the value to latest. In production, the versions will always be the same
    if [ "$IS_LOCAL" == "true" ]; then
        JFI_CORE_VERSION="${JFMC_VERSION}"
        JFI_SCHEDULER_VERSION="${JFMC_VERSION}"
        JFI_EXECUTOR_VERSION="${JFMC_VERSION}"
    else
        JFI_CORE_VERSION=$JFMC_VERSION
        JFI_SCHEDULER_VERSION=$JFMC_VERSION
        JFI_EXECUTOR_VERSION=$JFMC_VERSION
    fi

    #In interactive mode, ask for the version, & if nothing is entered, take the original value
    if [ "$JFI_INTERACTIVE" == true ]; then

        local temp_var=$JFI_CORE_VERSION
        read -p "Enter core version [${JFI_CORE_VERSION}]: " JFI_CORE_VERSION
        : ${JFI_CORE_VERSION:=$temp_var}
        
        local temp_var=$JFI_SCHEDULER_VERSION
        read -p "Enter scheduler version [${JFI_SCHEDULER_VERSION}]: " JFI_SCHEDULER_VERSION
        : ${JFI_SCHEDULER_VERSION:=$temp_var}

        local temp_var=$JFI_EXECUTOR_VERSION
        read -p "Enter executor version [${JFI_EXECUTOR_VERSION}]: " JFI_EXECUTOR_VERSION
        : ${JFI_EXECUTOR_VERSION:=$temp_var}
    fi

    DOCKER_SERVER_CURRENT_VER=$(docker version --format '{{.Server.Version}}') || DOCKER_SERVER_CURRENT_VER=""

    # Mission Control installer workdir
    JFMC_DIR="${HOME}/.jfrog/jfmc/jfmc-installer"

    # Mission Control installer conf file, should be created in the first run
    JFMC_CONF="${JFMC_DIR}/jfmc-env.conf"
    JFMC_MOUNT_ROOT_DEFAULT="${HOME}/.jfrog/jfmc"
    JFMC_PORT="8080"

    if [ ! -d "$JFMC_DIR" ]; then
        mkdir -p "$JFMC_DIR"
    fi
}

createInstallerConfigFile() {
    # Create the installer configuration file
    if [ ! -f "$JFMC_CONF" ]; then
        echo -e "\nJFMC_MOUNT_ROOT must be set on jfmc first installation \nPlease type desired JFMC_MOUNT_ROOT location"

        read -p "Enter mount root [${JFMC_MOUNT_ROOT_DEFAULT}]: " JFMC_MOUNT_ROOT
        JFMC_MOUNT_ROOT=${JFMC_MOUNT_ROOT:-${JFMC_MOUNT_ROOT_DEFAULT}}
        # Write to conf file
        echo "export JFMC_MOUNT_ROOT=${JFMC_MOUNT_ROOT}" >> ${JFMC_CONF}
        echo "JFMC_MOUNT_ROOT is now specified in ${JFMC_CONF}, as ${JFMC_MOUNT_ROOT}"
    fi
}

loadInstallerConfigFile() {
    # Load the installer configuration file
    if [ -f "$JFMC_CONF" ]; then
        source "${JFMC_CONF}"
    else
        echo "ERROR: configuration file does not exist [${JFMC_CONF}]"
        exit 1
    fi

    if [ -z ${JFMC_MOUNT_ROOT} ]
    then
        echo "ERROR: JFMC_MOUNT_ROOT is not set, please update the conf file [${JFMC_CONF}]"
        exit 1
    fi
    echo "INFO: Using JFMC_MOUNT_ROOT=${JFMC_MOUNT_ROOT}"

    ETC_FILE=${JFMC_MOUNT_ROOT}/etc/mission-control.properties
}

setupDockerParameters() {
    # Pass an overwritten port
    if [ -f ${ETC_FILE} ]; then
        grep "^server.port" ${ETC_FILE}
        if [ $? -eq 0 ]; then
            JFMC_PORT=$(grep  "^server.port" ${ETC_FILE} | awk -F'=' '{print $2}')
        fi
    fi

    # Setup options for connecting to docker host
    if [ -z "$DOCKER_HOST" ]; then
        DOCKER_HOST="/var/run/docker.sock"
    fi
    if [ -S "$DOCKER_HOST" ]; then
        DOCKER_ADDR="-v $DOCKER_HOST:$DOCKER_HOST -e DOCKER_HOST"
    else
        DOCKER_ADDR="-e DOCKER_HOST -e DOCKER_TLS_VERIFY -e DOCKER_CERT_PATH"
    fi

    # Only allocate tty if we detect one
    if [ -t 1 ]; then
        DOCKER_RUN_OPTIONS="-t"
    fi
    if [ -t 0 ]; then
        DOCKER_RUN_OPTIONS="$DOCKER_RUN_OPTIONS -i"
    fi

    JFMC_VOLUMES="-v ${JFMC_MOUNT_ROOT}/jfmc-installer:/data -v ${JFMC_MOUNT_ROOT}:/jfmc_global_mount_root"

    JFMC_DOCKER_REPO="%%dockerRepo%%"

    IMAGE="${JFMC_DOCKER_REPO}/jfrog/jfmc-installer:$JFMC_VERSION"

    [ "${JFMC_PORT}" != "8080" ] && echo "INFO: Using none default http port: ${JFMC_PORT}"

    if [ "$IS_LOCAL" == "true" ]; then
        # Before running, try to login to the registry if login password is provided
        echo "Going to docker login ${JFMC_DOCKER_REPO}...."
        docker login -u ${JFMC_DOCKER_REPO_LOGIN} -p ${JFMC_DOCKER_REPO_PASSWORD} $JFMC_DOCKER_REPO
        JFMC_EXTRA_LOGIN_PARAMS="-e JFMC_DOCKER_REPO_LOGIN=$JFMC_DOCKER_REPO_LOGIN -e JFMC_DOCKER_REPO_PASSWORD=$JFMC_DOCKER_REPO_PASSWORD"
    fi
}

##################################
############## MAIN ##############
##################################

setUp
createInstallerConfigFile
loadInstallerConfigFile
setupDockerParameters

exec docker run --name "jfmc_installer" --rm $DOCKER_RUN_OPTIONS $DOCKER_ADDR $JFMC_VOLUMES $JFMC_EXTRA_LOGIN_PARAMS \
-e JFMC_PORT=$JFMC_PORT \
-e JFMC_DOCKER_REPO=$JFMC_DOCKER_REPO -e JFMC_VERSION=$JFMC_VERSION -e JFI_CORE_VERSION=$JFI_CORE_VERSION \
-e JFI_EXECUTOR_VERSION=$JFI_EXECUTOR_VERSION -e JFI_SCHEDULER_VERSION=$JFI_SCHEDULER_VERSION -e JFMC_MOUNT_ROOT=$JFMC_MOUNT_ROOT \
-e DOCKER_SERVER_CURRENT_VER=$DOCKER_SERVER_CURRENT_VER $IMAGE "$CMD"

exit $?
